<?php
/*~ serverlogs.php
.----------------------------------------------------------------------------.
|  Software: serverlogs - view text server logs last records for phpBB admin |
|      Site: http://c61.su/forum/index.php                                   |
| -------------------------------------------------------------------------- |
|     Author: Serg c61@yandex.ru                                             |
| Copyright (c) 2015, Serg c61@yandex.ru. All Rights Reserved.               |
| -------------------------------------------------------------------------- |
|   License: GNU General Public License v2                                   |
|            http://opensource.org/licenses/gpl-2.0.php                      |
'----------------------------------------------------------------------------'*/

// Version:
$serverlogs_version = '1.0.0';

//------------------------------------------------------------------------------
//
// c61@yandex.ru : Просмотр последних записей текстовых логов сервера админом phpBB
// v1.0.0  : 20151103 : первый релиз
//
//------------------------------------------------------------------------------
//
// phpBB
//

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);

if ( !$auth->acl_get('a_') )
{
	trigger_error('NO_AUTH_ADMIN');
}

//------------------------------------------------------------------------------
//
$logs = array();
//
// Задайте здесь список логов
$logs[] = array(
					'name'	=> 'PHP error log',
					'file'	=> 'C:\Server\web\apache\logs\error.log',
					'last'	=> 100,
				);
$logs[] = array(
					'name'	=> 'Apache access log',
					'file'	=> 'C:\Server\web\apache\logs\access.log',
					'last'	=> 100,
				);

// Обратная шкала времени (новые записи лога вверху)
$reverse = true;

//------------------------------------------------------------------------------
//
// Функция чтения $nlines текстовых строк с конца файла $filename
//

function ReverseReadFile($filename, $nlines, $bufferlength = 4096)
{
	if ( ($bufferlength <= 0) || ($nlines <= 0) )
	{
		return false;
	}

	if ( false === ($handle = @fopen($filename, "rb")) )
	{
		return false;
	}

	if ( false === @fseek($handle, 0, SEEK_END) )
	{
        fclose($handle);
		return false;
	}

	if ( false === ($filesize = @ftell($handle)) )
	{
        fclose($handle);
		return false;
	}

	$position = -min($bufferlength, $filesize);
	$lines = array();
	$remainder = false;

	while ( $nlines > 0 )
	{
		if ( false === @fseek($handle, $position, SEEK_END) )
		{
	        fclose($handle);
			return false;
		}

		if ( false === ($buffer = @fread($handle, $bufferlength)) )
		{
	        fclose($handle);
			return false;
		}

		$tmp = explode("\n",$buffer);
		if ( false !== $remainder )
		{
			$tmp[count($tmp)-1] .= $remainder;
		}
		$remainder = array_shift($tmp);

		$read = count($tmp);
		if ( $read >= $nlines )
		{
			$lines = array_merge(array_slice($tmp, $read-$nlines), $lines);
			break;
		}

		$lines = array_merge($tmp, $lines);
		$nlines -= $read;
		$position -= $bufferlength;

		if ( -$position >= $filesize )
		{
			$position += $bufferlength;
			$bufferlength = $filesize + $position;
			if ( !$bufferlength )
			{
				if ( false !== $remainder ) $lines = array_merge(array($remainder), $lines);
				break;
			}
			$position = -$filesize;
		}
	}
	fclose($handle);

	return $lines;
}

//------------------------------------------------------------------------------
//
// Параметры

if (PHPBB_VERSION >= '3.1.0')
{
	$self = $request->server('PHP_SELF');
}
else
{
	$self = $_SERVER['PHP_SELF'];
}

//------------------------------------------------------------------------------
//
// Page header
//
@header( 'Content-type: text/html; charset=utf-8' );

echo '<!DOCTYPE html>
<html dir="ltr" lang="ru">

<head>
<meta charset="utf-8" />
<title>Server Logs</title>
<style type="text/css">
body {
	font-family: "Courier New", monospace;
}
a {
	text-decoration: none;
	font-weight: bold;
	font-style: italic;
}
#log {
	width:"100%";
	max-width: "100%";
	overflow-x: scroll;
	font: inherit;
	margin: 0;
	padding: 0;
}
pre {
	font: inherit;
	margin: 0;
	padding: 0;
	white-space: pre-wrap;
	white-space: -moz-pre-wrap;
	white-space: -pre-wrap;
	white-space: -o-pre-wrap;
	word-wrap: break-word;
	-o-tab-size: 4;
	-moz-tab-size: 4;
	-webkit-tab-size: 4;
	tab-size: 4;
}
code {
	font: inherit;
	margin: 0;
	padding: 0;
	white-space: inherit;
	word-wrap: inherit;
	tab-size: inherit;
}
p {
	margin: 0 4px 5px 4px;
	padding: 0;
}
</style>
</head>

<body>
<h3>Просмотр последних записей текстовых логов сервера админом phpBB</h3>
';
//------------------------------------------------------------------------------
//
// Page body
//
for ( $i=0; $i<count($logs); $i++ )
{
	echo ($i+1) . ') <a href="' . $self . '?i=' . $i . '">' . htmlspecialchars($logs[$i]['name']) . '</a><br />
';
}

$i = request_var('i',-1);

if ( isset($logs[$i])
 && (false !== ($strings = ReverseReadFile($logs[$i]['file'], $logs[$i]['last'])))
 && count($strings)
	)
{
	echo '<br /><hr></hr><h3>Последние ' . $logs[$i]['last'] . ' записей лога "' . $logs[$i]['name'] . '":</h3><div id="log"><pre><code>';

	if ( $reverse ) $strings = array_reverse($strings);

	foreach ($strings as $string)
	{
		// Выделение
		foreach (array(']','=',': ',' ') as $needle)
		{
			if ( false !== ($pos = strpos($string,$needle)) )
			{
				$string = '<b>' . substr($string,0,$pos+1) . '</b>' . substr($string,$pos+1);
				break;
			}
		}
		// Удаляем \r
		$string = '<p>' . str_replace("\r",'',$string) . '</p>';
		echo $string;
	}
	echo '</code></pre></div>
';
}

//------------------------------------------------------------------------------
//
// Page footer
//
echo '<div width="100%" align="right"><small><i><a href="http://c61.su" target="_blank">serverlogs</a> v.' . $serverlogs_version . ' (C) 2015 <a href="mailto:c61@yandex.ru">c61</a></i></small></div>
</body>
</html>
';
?>